/*
ͻ
 What's it called: TeX Front End key definitions (xtexfek.e), v3.1,         
                   with extended LaTeX syntax assist                        
                                                                            
 What does it do:  Defines the TeX keysets and syntax assist procedures.    
                   (linked in automatically by texfe.e)                     
                                                                            
 Who and When:     Jo Koenen 7/94                                           
                   Jon Hacker 8/95                                          
                   Walter Schmidt  2/3002                                   
                   Juan J.G. Ripoll 3/1998                                  
ͼ
*/

--  Since this file is distributed separately from texfe, we must ensure
--  that it is compiled in conjunction with the appropriate texfe version:

compile if PrimVersion < '2.0' -- changed 2/2002
  *** Error: This version of xtexfek.e is only for texfe 2.0 and above
compile endif

compile if not defined(ENTER_ACTION)
   const ENTER_ACTION = 'ADDLINE'
compile endif

compile if not defined(ENHANCED_ENTER_KEYS)
   const ENHANCED_ENTER_KEYS = 1 -- ???
compile endif

/*
Ŀ
  Tex keysets are loaded during defselect                                   
  We put the keys statement in the defselect so that we can check           
  to make sure it is a TeX file before we load the keyset.                  

*/

defproc setTeXKeySets
    universal app_hini

    if tex_is_doc() or tex_is_style() then  -- if TeX doc/style loaded
        if queryprofile( app_hini, TEX_APP, 'TEX_KEYS') then
            if translate(.keyset) <> 'TEXKEYSA' then keys TEXKEYSA endif
        else
            if translate(.keyset) <> 'TEXKEYSN' then keys TEXKEYSN endif
        endif -- TEX_KEYS
    elseif tex_is_aux() then
        if translate(.keyset) <> 'TEXKEYSQ' then keys TEXKEYSQ endif
    endif  -- TeX doc/style loaded


/*
Ŀ
  Define the TeX syntax assist key set combo                                

*/

defkeys TEXKEYSA   -- quick keys + syntax expansion + EPMTeX accelerator keys 
define TEXACCEL = 1
define TEXCOMPL = 1
include 'xtexfed.e'

defkeys TEXKEYSN   -- quick keys + syntax expansion
define TEXACCEL = 0
define TEXCOMPL = 1
include 'xtexfed.e'

defkeys TEXKEYSQ   -- quick keys only
define TEXACCEL = 0
define TEXCOMPL = 0
include 'xtexfed.e'

/*
ͻ
 TeX syntax assist procedure definitions.                                   
                                                                            
 What does it do:  The enter and space bar keys have been defined to have   
                   specific TeX editing features in this keyset.            
                                                                            
ͼ
*/

  defproc tex_first_expansion
    retc=1   -- default return code
  compile if EVERSION >= 5
    if .line then  -- current line number
  compile else
    if .line and (not command_state()) then
  compile endif
      getline line
      line=strip(line,'T') -- remove trailing spaces
      line_l=substr(line,1,.col-1) -- split line into two parts at cursor
      line_r=substr(line,.col)
      linelen=length(line_l)
      if linelen > 2 then
        wrd=substr(line_l,.col-3,3)
      else
        wrd=line_l
      endif
-- expand \begin
      if wrd='\be' then
        replaceline line_l'gin{}'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+4
-- expand \cite
      elseif wrd='\ci' then
        replaceline line_l'te{}'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+3  -- reposition cursor
-- expand \label
      elseif wrd='\la' then
        replaceline line_l'bel{}'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+4  -- reposition cursor
-- expand \ref
      elseif wrd='\re' then
        replaceline line_l'f{}'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+2  -- reposition cursor
-- expand \fraction
      elseif wrd='\fr' then
        replaceline line_l'ac{}{}'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+3  -- reposition cursor
-- expand equation
      elseif wrd='equ' then
        replaceline line_l'ation'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+6  -- reposition cursor
-- expand eqnarray
      elseif wrd='eqn' then
        replaceline line_l'array'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+6  -- reposition cursor
-- expand footnote
      elseif wrd='\fo' then
        replaceline line_l'otnote{}'line_r
        if not insert_state() then insert_toggle
        compile if EVERSION >= '5.50'
          call fixup_cursor()
        compile endif
        endif
        .col=.col+7  -- reposition cursor
-- future expansions added here
      else  -- wrd =
        retc=0  -- didn't match wrd, no expansion performed
      endif
    else   -- .line  at top of file, abort expansion
      retc=0
    endif  -- .line = 0
  return retc

/*
Ŀ
  Define second TeX syntax expansion procedure for spacebar and enter       
    -- returns 1 if expansion performed                                     

*/
  defproc tex_second_expansion
    universal TeX_Margin
    retc=1  -- default return value
    if .line then
      getline lin
      if (.col - 21) < 0 then
        b=1
      else
        b=.col-21+1
      endif
      line=substr(lin,b,.col-b+1)  -- get last 21 chars from line end
      parse value line with '\'wrd'{'texword'}'
-- add \end if find \begin keyword
      if wrd='begin' then
        if texword='tabular' then
          line_l=substr(lin,1,.col)
          line_r=substr(lin,.col+1)
          replaceline line_l'{|c|}'line_r
        endif
-- We move to the beginning of the next line
-- insert an empty line, insert an \end{texword} line
-- indent it and finally move up to the empty line
	.col = 0
	insertline '\end{'texword'}', .line + 1
	insertline '', .line + 1
	call LaTeX_indent_line2(TeX_Margin)
	.lineg = .lineg + 2
	call LaTeX_indent_line2(TeX_Margin)
	.lineg = .lineg - 1
      else
        retc=0
      endif  -- wrd =
    else  -- .line =
      retc=0
    endif -- .line =
-- Always indent the current line
    call LaTeX_indent_line(TeX_Margin, 0)
  return retc

/* changed 1997-12-12: analyze 21 chars */

/* changed 1998-03-08 for v2.0:
   expand \fo -> \footnote{},
   version check, 
   always define ENTER_ACTION, 
   input xtexfed.e
*/
/* changed 1998-03-23 call LaTeX_Indent_line with `advance' parameter */


